package com.huigou.topsun.sap.qualityInfoRecord.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.huigou.cache.SystemCache;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.common.HttpClient;
import com.huigou.topsun.sap.common.domain.SapResult;
import com.huigou.topsun.sap.qualityInfoRecord.application.SapQualityInfoRecordApplication;
import com.huigou.topsun.sap.qualityInfoRecord.application.SapQualityInfoRecordItemApplication;
import com.huigou.topsun.sap.qualityInfoRecord.domain.SapQualityInfoRecord;
import com.huigou.topsun.sap.qualityInfoRecord.domain.SapQualityInfoRecordItem;
import com.huigou.topsun.sap.qualityInfoRecord.domain.query.SapQualityInfoRecordQueryRequest;
import com.huigou.topsun.sap.qualityInfoRecord.domain.vo.SapQualityInfoRecordItemVo;
import com.huigou.topsun.sap.qualityInfoRecord.repository.SapQualityInfoRecordRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Auther: xin.lu
 * @Date: 2024/02/28/15:40
 * @Description:
 */
@Service("sapQualityInfoRecordApplication")
public class SapQualityInfoRecordApplicationImpl extends FlowBroker implements SapQualityInfoRecordApplication {

    @Autowired
    private SapQualityInfoRecordRepository sapQualityInfoRecordRepository;
    @Autowired
    private SapQualityInfoRecordItemApplication sapQualityInfoRecordItemApplication;
    @Resource
    private HttpClient httpClient;
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        SapQualityInfoRecord sapQualityInfoRecord = getBizEntity(SapQualityInfoRecord.class);
        if (sapQualityInfoRecord.isNew()) {
            sapQualityInfoRecord.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            sapQualityInfoRecord = (SapQualityInfoRecord) commonDomainService.loadAndFillinProperties(sapQualityInfoRecord);
        }
        sapQualityInfoRecord = sapQualityInfoRecordRepository.save(sapQualityInfoRecord);
        List<SapQualityInfoRecordItem> sapQualityInfoRecordItems = getBizEntities(SapQualityInfoRecordItem.class, "items");
        sapQualityInfoRecordItemApplication.saveSapQualityInfoRecordItem(sapQualityInfoRecord.getId(), sapQualityInfoRecordItems);

        return sapQualityInfoRecord.getId();
    }



    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(sapQualityInfoRecordRepository.getOne(bizId));
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SapQualityInfoRecord sapQualityInfoRecord = sapQualityInfoRecordRepository.findOne(bizId);
        sapQualityInfoRecord.setStatusId(status.getId());
        sapQualityInfoRecordRepository.save(sapQualityInfoRecord);

        this.getSapPurchaseInfoRecordItemVo(sapQualityInfoRecord);
    }

    public void getSapPurchaseInfoRecordItemVo(SapQualityInfoRecord sapQualityInfoRecord){
        List<SapQualityInfoRecordItem> purchaseInfoRecordItemList = sapQualityInfoRecordItemApplication.findBySapQualityInfoRecordId(sapQualityInfoRecord.getId());
        List<SapQualityInfoRecordItemVo> sapQualityInfoRecordItemVoList = new ArrayList<>();
        purchaseInfoRecordItemList.forEach(sapQualityInfoRecordItem -> {
            SapQualityInfoRecordItemVo sapQualityInfoRecordItemVo = new SapQualityInfoRecordItemVo();
            BeanUtil.copyProperties(sapQualityInfoRecordItem, sapQualityInfoRecordItemVo);

            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
            if (sapQualityInfoRecordItem.getGrscertdatai() !=null){
                String datab = dateFormat.format(sapQualityInfoRecordItem.getGrscertdatai());
                sapQualityInfoRecordItemVo.setGrscertdatai(datab);
            }
            if (sapQualityInfoRecordItem.getFsccertdatai() != null){
                String datbi = dateFormat.format(sapQualityInfoRecordItem.getFsccertdatai());
                sapQualityInfoRecordItemVo.setFsccertdatai(datbi);
            }
            if (sapQualityInfoRecordItem.getZvdatab() != null){
                String zvdatab = dateFormat.format(sapQualityInfoRecordItem.getZvdatab());
                sapQualityInfoRecordItemVo.setZvdatab(zvdatab);
            }
            if (sapQualityInfoRecordItem.getZvdatai() != null){
                String zvdatai = dateFormat.format(sapQualityInfoRecordItem.getZvdatai());
                sapQualityInfoRecordItemVo.setZvdatai(zvdatai);
            }
            if (sapQualityInfoRecord.getFillinDate() != null){
                String fillinDate = dateFormat.format(sapQualityInfoRecord.getFillinDate());
                sapQualityInfoRecordItemVo.setErstelldat(fillinDate);
            }
            sapQualityInfoRecordItemVo.setErsteller(sapQualityInfoRecord.getPersonMemberName());
            sapQualityInfoRecordItemVoList.add(sapQualityInfoRecordItemVo);
        });
        try {
            String url = SystemCache.getParameter("sap.QualityInfoRecord.url", String.class);
            String execute = httpClient.execute(sapQualityInfoRecordItemVoList, url);
            List<SapResult> resultList = JSONObject.parseArray(execute, SapResult.class);
            for (SapResult sapResult : resultList) {
                if (!"S".equals(sapResult.getTYPE())){
                    throw new RuntimeException("数据传输失败，"+ sapResult.getMESSAGE());
                }
                for (SapQualityInfoRecordItem sapQualityInfoRecordItem : purchaseInfoRecordItemList) {
                    if (sapResult.getMESSAGE_V1().equals(sapQualityInfoRecordItem.getZqinfno())){
                        sapQualityInfoRecordItem.setType(sapResult.getTYPE());
                        sapQualityInfoRecordItem.setMessage(sapResult.getMESSAGE());
                        sapQualityInfoRecordItemApplication.save(sapQualityInfoRecordItem);
                    }
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SapQualityInfoRecord sapQualityInfoRecord = sapQualityInfoRecordRepository.findOne(bizId);
        sapQualityInfoRecord.setStatusId(BizBillStatus.ABORTED.getId());
        sapQualityInfoRecordRepository.save(sapQualityInfoRecord);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SapQualityInfoRecord sapQualityInfoRecord = sapQualityInfoRecordRepository.findOne(bizId);
            sapQualityInfoRecord.setStatusId(BizBillStatus.APPLYING.getId());
            sapQualityInfoRecordRepository.save(sapQualityInfoRecord);
        }
    }

    @Override
    public Map<String, Object> slicedSapQualityInfoRecordList(SapQualityInfoRecordQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "sapQualityInfoRecordList");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.putDictionary("status", BizBillStatus.getMap());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public SapQualityInfoRecord loadSapQualityInfoRecord(String id) {
        return sapQualityInfoRecordRepository.findOne(id);
    }

}
